import hashlib
import os


class Meta:
    def __init__(self, filepath, buf_size=4 * 1024 * 1024):
        abs_path = os.path.abspath(filepath)
        if not os.path.exists(abs_path):
            print(f"文件不存在 请检查: {abs_path}")
            exit(-1)

        self.buf_size = buf_size

        self._path = abs_path
        self._basename = os.path.basename(filepath)
        self._size = os.path.getsize(filepath)
        self._sha1 = None
        self._md5 = None
        self._115_head_hash = None
        self._baidu_head_hash = None

    def gen_115_link(self):
        sha1, _ = self.get_sha1_and_md5()
        head_sha1 = self.get_115_head_hash()
        return f"115://{self._basename}|{self._size}|{sha1}|{head_sha1}"

    def gen_ali_link(self):
        sha1, _ = self.get_sha1_and_md5()
        return f"aliyunpan://{self._basename}|{sha1}|{self._size}|TMP"

    def save_meta(self):
        with open(self._path + '.licsber.csv', 'w') as f:
            f.write(str(self))

    def get_basename(self):
        return self._basename

    def get_size(self):
        return self._size

    def get_sha1_and_md5(self):
        def cal():
            sha1_obj = hashlib.sha1()
            md5_obj = hashlib.md5()
            with open(self._path, 'rb') as f:
                while True:
                    content = f.read(self.buf_size)
                    if not content:
                        break

                    sha1_obj.update(content)
                    md5_obj.update(content)

            self._sha1 = sha1_obj.hexdigest().upper()
            self._md5 = md5_obj.hexdigest().upper()

        if not self._sha1 or not self._md5:
            cal()

        return self._sha1, self._md5

    def get_115_head_hash(self):
        def cal():
            with open(self._path, 'rb') as f:
                sha1_obj = hashlib.sha1()
                num_bytes = 128 * 1024
                content = f.read(num_bytes)
                if (l := len(content)) < num_bytes:
                    content += b'\0' * (num_bytes - l)

                sha1_obj.update(content)
                self._115_head_hash = sha1_obj.hexdigest().upper()

        if not self._115_head_hash:
            cal()

        return self._115_head_hash

    def get_baidu_head_hash(self):
        raise Exception('暂时还没做')

    def meta(self):
        self.gen_115_link()
        return self._basename, self._size, self._sha1, self._115_head_hash, self._md5

    def __str__(self):
        res = 'Key,Filename,Size,SHA1,HeadSHA1,MD5\n'
        sha1, md5 = self.get_sha1_and_md5()
        head_sha1 = self.get_115_head_hash()
        res += f"Value,{self._basename},{self._size},{sha1},{head_sha1},{md5}\n"
        res += f",,,,,{self.gen_115_link()}\n"
        res += f",,,,,{self.gen_ali_link()}\n"
        return res


if __name__ == '__main__':
    test_path = '/tmp/test.licsber'
    with open(test_path, 'w') as f:
        f.write('Hello Licsber.')

    meta = Meta(test_path)
    meta.save_meta()
    print(meta, end='')
    assert meta.gen_115_link() == '115://test.licsber|14|02B02681636CCEDB820385C8A87EA2E1E18ACD5C|C24486ADE0E6AAE9376E4994A7A1267277A13295'



import cv2
import matplotlib.pyplot as plt


def imshow(img):
    """
    转换奇怪的bgr和rgb
    :param img: numpy array
    """
    if len(img.shape) != 3:
        plt.imshow(img)
        return

    b, g, r = cv2.split(img)
    rgb = cv2.merge([r, g, b])
    plt.imshow(rgb)



import base64
import random

import requests
from Crypto.Cipher import AES

from licsber.utils.utime import timestamp_mil

CAPTCHA_URL = 'http://authserver.njit.edu.cn/authserver/captcha.html'


def pad(s: bytes, block_size: int) -> bytes:
    l = len(s)
    pad_num = block_size - (l % block_size)
    if pad_num == 0:
        pad_num = block_size
    pad_b = bytes([pad_num])
    return s + pad_b * pad_num


def aes_encrypt(s: str, key: str, iv='\0' * 16, coding='utf-8') -> str:
    key_b = key.encode(coding)
    iv_b = iv.encode(coding)
    raw_b = s.encode(coding)

    cipher = AES.new(key_b, AES.MODE_CBC, iv_b)
    padded = pad(raw_b, AES.block_size)
    encrypted = cipher.encrypt(padded)
    encoded = base64.b64encode(encrypted)
    return encoded.decode(coding)


def encrypt(pwd, salt):
    charsets = 'ABCDEFGHJKMNPQRSTWXYZabcdefhijkmnprstwxyz2345678'
    rnd_16 = ''.join(random.choice(charsets) for _ in range(16))
    rnd_64 = ''.join(random.choice(charsets) for _ in range(64))
    return aes_encrypt(rnd_64 + pwd, salt, rnd_16)


def check_captcha(text):
    SIMPLE_CHAR_LIST = '134578ABCDEFHKNPQXYcfkxy'

    if len(text) != 4:
        return False

    for ch in text:
        if ch not in SIMPLE_CHAR_LIST:
            return False

    return True


def get_captcha(url: str, session: requests.Session):
    CAPTCHA_URL = url.split('/authserver/')[0] + '/authserver/captcha.html'
    params = {
        'ts': random.randint(1, 999),
    }
    res = session.get(CAPTCHA_URL, params=params)
    return res.content


def need_captcha(url: str, session: requests.Session, no):
    NEED_CAPTCHA_URL = url.split('/authserver/')[0] + '/authserver/needCaptcha.html'
    params = {
        'username': no,
        'pwdEncrypt2': 'pwdEncryptSalt',
        '_': timestamp_mil(),
    }
    res = session.get(NEED_CAPTCHA_URL, params=params)
    return res.json()



import cv2
import numpy as np


def parse_img(inputs):
    """
    将输入转化为图片, 支持路径/numpy array/content(图片二进制).
    :param inputs: 任意输入
    :return: numpy array
    """
    if type(inputs) == str:
        return cv2.imread(inputs)
    elif type(inputs) == bytes:
        return cv2.imdecode(np.frombuffer(inputs, np.uint8), cv2.COLOR_RGBA2RGB)
    else:
        return inputs



import functools
import inspect
import os
import sys


def check_path_exist(start=None):
    if not start:
        start = os.getcwd() if len(sys.argv) == 1 else os.path.join(os.getcwd(), sys.argv[1])

    if not os.path.exists(start):
        print(f'目录 {start} 不存在, 请检查.')
        exit(-1)

    start = os.path.abspath(start)
    return start


def clean_macos(start):
    for root, dirs, files in os.walk(start):
        for i in files:
            filepath = os.path.join(root, i)
            if not os.path.isfile(filepath):
                continue

            meta_path = os.path.join(root, '._' + i)
            if i == '.DS_Store':
                os.remove(filepath)
                print(f"删除.DS_Store: {root}")
            elif os.path.exists(meta_path):
                meta = open(meta_path, 'rb').read()
                if b'This resource fork intentionally left blank' in meta:
                    os.remove(meta_path)
                    print(f"删除元文件: {meta_path}")


def fun_check_path_exist(arg_name='start_path', clean=False, print_path=True):
    def decorator(fun):
        @functools.wraps(fun)
        def wrapper(*args, **kwargs):
            sign = inspect.signature(fun)

            # Not using sign.bind_partial() because in this decorator,
            # we dont want arguments be loss except None
            # 不用sign.bind_partial() 如果缺少参数报ValueError而不是添加
            bound_arguments = sign.bind(*args, **kwargs)
            path = bound_arguments.arguments.get(arg_name)
            insert = path is None
            path = check_path_exist(path)

            if insert:
                kwargs[arg_name] = path

            if print_path:
                print(f'传入目录: {path}')

            if clean:
                clean_macos(path)

            return fun(*args, **kwargs)

        return wrapper

    return decorator


def all_filepath(start_path):
    for filename in os.listdir(start_path):
        filepath = os.path.join(start_path, filename)
        if os.path.isfile(filepath):
            yield filepath


def walk_files(res: list, start_path):
    """
    传入一个list, 递归遍历start_path, 向list中添加文件路径.
    """
    for i in os.listdir(start_path):
        path = os.path.join(start_path, i)
        if os.path.isdir(path):
            walk_files(res, path)
        else:
            res.append(path)


@fun_check_path_exist(arg_name='save_path', print_path=False)
def save_file(save_path, filename, text):
    """
    忽略这个函数 库中代码减少行数使用 也为了方便调试
    """
    with open(os.path.join(save_path, filename), 'w') as f:
        f.write(text)



import os
import smtplib
from email.header import Header
from email.mime.multipart import MIMEMultipart
from email.mime.text import MIMEText

from licsber.utils.utime import localtime_date_str


def _parse_dict(detail: dict) -> str:
    res = ''
    for i in detail:
        res += str(i) + ': ' + str(detail[i]) + '<br>'
    return res


class SMTP:
    def __init__(self, password,
                 mail_address,
                 smtp_server='smtp.qq.com',
                 sender=None,
                 username=None):
        if not sender:
            sender = mail_address

        if not username:
            username = mail_address

        self.sender = sender
        self.smtp = smtplib.SMTP()

        self.smtp.connect(smtp_server)
        self.smtp.login(username, password)

    def plain_mail_to(self, title: str, body: str,
                      receiver=None,
                      mail_from='Licsber Automatic'):
        if not receiver:
            receiver = self.sender
        msg = MIMEText(body, 'plain', 'utf-8')
        msg["Accept-Language"] = "zh-CN"
        msg["Accept-Charset"] = "ISO-8859-1,utf-8"
        msg['From'] = mail_from
        msg['To'] = receiver
        msg['Subject'] = Header(title, 'utf-8')

        try:
            self.smtp.sendmail(self.sender, receiver, msg.as_string())
            return True
        except smtplib.SMTPException:
            return False

    def notice_mail_to(self, mail_title: str, title: str, detail: any,
                       receiver=None,
                       mail_from='Licsber Automatic'):
        if not receiver:
            receiver = self.sender

        if type(detail) is dict:
            detail = _parse_dict(detail)

        mail_title = localtime_date_str() + ' ' + mail_title

        msg = MIMEMultipart('alternative')

        msg["Accept-Language"] = "zh-CN"
        msg["Accept-Charset"] = "ISO-8859-1,utf-8"
        msg['From'] = mail_from
        msg['To'] = receiver
        msg['Subject'] = Header(mail_title, 'utf-8')
        template_path = os.path.join(os.path.dirname(__file__), 'notice_template.html')
        html = open(template_path).read()
        html = html.replace('{{title}}', title)
        html = html.replace('{{detail}}', detail)

        html = MIMEText(html, 'html')
        msg.attach(html)

        try:
            self.smtp.sendmail(self.sender, receiver, msg.as_string())
            return True
        except smtplib.SMTPException:
            return False
